#ifndef LIBRESIDFP_VERSION_H
#define LIBRESIDFP_VERSION_H

#ifndef RESIDFP_H
#  error Do not include directly.
#endif



#define LIBRESIDFP_VERSION_MAJ 1
#define LIBRESIDFP_VERSION_MIN 2
#define LIBRESIDFP_VERSION_LEV 2

#endif
